@@ -298,8 +298,8 @@ add_underline (PangoRenderer    *renderer,
   int underline_thickness = pango_font_metrics_get_underline_thickness (metrics);
   int underline_position = pango_font_metrics_get_underline_position (metrics);
 
-  new_rect.x = base_x + ink_rect->x;
-  new_rect.width = ink_rect->width;
+  new_rect.x = base_x + logical_rect->x;
+  new_rect.width = logical_rect->width;
   new_rect.height = underline_thickness;
   new_rect.y = base_y;
   
@@ -322,9 +322,7 @@ add_underline (PangoRenderer    *renderer,
       new_rect.y == current_rect->y &&
       new_rect.height == current_rect->height)
     {
-      current_rect->y = new_rect.y;
       current_rect->width = new_rect.x + new_rect.width - current_rect->x;
-      current_rect->height = new_rect.height;
     }
   else
     {
@@ -350,8 +348,8 @@ add_strikethrough (PangoRenderer    *renderer,
   int strikethrough_thickness = pango_font_metrics_get_strikethrough_thickness (metrics);
   int strikethrough_position = pango_font_metrics_get_strikethrough_position (metrics);
   
-  new_rect.x = base_x + ink_rect->x;
-  new_rect.width = ink_rect->width;
+  new_rect.x = base_x + logical_rect->x;
+  new_rect.width = logical_rect->width;
   new_rect.y = base_y - strikethrough_position;
   new_rect.height = strikethrough_thickness;
   
@@ -359,9 +357,7 @@ add_strikethrough (PangoRenderer    *renderer,
       new_rect.y == current_rect->y &&
       new_rect.height == current_rect->height)
     {
-      current_rect->y = new_rect.y;
       current_rect->width = new_rect.x + new_rect.width - current_rect->x;
-      current_rect->height = new_rect.height;
     }
   else
     {
@@ -450,6 +446,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 				 int               y)
 {
   int x_off = 0;
+  int glyph_string_width;
   LineState state;
   GSList *l;
   gboolean got_overall = FALSE;
@@ -480,8 +477,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
       gint rise;
       PangoLayoutRun *run = l->data;
       PangoAttrShape *shape_attr;
-      PangoRectangle logical_rect;
-      PangoRectangle ink_rect;
+      PangoRectangle ink_rect, logical_rect;
 
       pango_renderer_prepare_run (renderer, run);
       
@@ -491,19 +487,22 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	{
 	  ink_rect = shape_attr->ink_rect;
 	  logical_rect = shape_attr->logical_rect;
+	  glyph_string_width = shape_attr->logical_rect.width;
 	}
       else
 	{
 	  if (renderer->underline != PANGO_UNDERLINE_NONE ||
 	      renderer->strikethrough)
-	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-					&ink_rect, &logical_rect);
+	    {
+	      pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
+					    &ink_rect, &logical_rect);
+	      glyph_string_width = logical_rect.width;
+	    }
 	  else
-	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-					NULL, &logical_rect);
+	    glyph_string_width = pango_glyph_string_get_width (run->glyphs);
 	}
 
-      state.logical_rect_end = x + x_off + logical_rect.x + logical_rect.width;
+      state.logical_rect_end = x + x_off + glyph_string_width;
 
       if (renderer->priv->color_set[PANGO_RENDER_PART_BACKGROUND])
 	{
@@ -515,9 +514,9 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
 	  
 	  pango_renderer_draw_rectangle (renderer,
 					 PANGO_RENDER_PART_BACKGROUND,
-					 x + x_off + logical_rect.x,
+					 x + x_off,
 					 y - rise + overall_rect.y,
-					 logical_rect.width,
+					 glyph_string_width,
 					 overall_rect.height);
 	}
 
@@ -558,7 +557,7 @@ pango_renderer_draw_layout_line (PangoRenderer    *renderer,
       if (!renderer->strikethrough && state.strikethrough)
 	draw_strikethrough (renderer, &state);
 
-      x_off += logical_rect.width;
+      x_off += glyph_string_width;
     }
 
   /* Finish off any remaining underlines
